# Copyright (c) 1995-1999 Nick Ing-Simmons. All rights reserved.
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.

package Gtk2::Simple::CascadeList;

our @ISA = qw/Gtk2::SimpleList Cascade/;
our $VERSION = '0.15';

use Data::Dumper;
use Carp;
use Gtk2;
use Gtk2::SimpleList;
use Cascade;

sub new {
	croak "Usage: $_[0]\->new (\\var, entry => value, ...)\n"
	    . " expecting a reference to the storage variable "
		. " and a list of entry and value pairs.\n"
	    . " can't create a SimpleGtk2:CascadeList with no entries"
		unless @_ >= 3; # class, key1, val1

	return shift->new_from_widget (Gtk2::TreeView->new (), @_);
}

sub new_from_widget {
  my $class = shift;
  my $widg = shift;
  croak "TreeView is not a Gtk2::TreeView"
	  unless defined ($widg)
		  and UNIVERSAL::isa ($widg, 'Gtk2::TreeView');
  croak "Usage: $class\->new_from_widget (treelist, title, \\var, entry => val, ...)\n"
	  . " expecting a TreeView reference, list title, a reference to the storage"
	  . " for the selected index"
		  unless @_ >= 2; # ref, key1, val1
  my $title = shift;
  my $var = shift;

  my $result = Gtk2::SimpleList->new_from_treeview($widg, $title => 'text');
  $result->get_selection
	  ->signal_connect(changed => 
					   sub { $result->SelectionCallback($title, @_); });

  $result->InitObject();
  return $result;
}


sub InitObject {
  my ($cw, $args) = @_;
  my $pvt = $cw->privateData();

#  $cw->Cascade::InitObject($args);
  Cascade::Initialize($cw, $cw);

  $cw->RegisterCallback("INSERT_CALLBACK", sub { InsertCallback($cw, @_); });
  $cw->RegisterCallback("DELETE_CALLBACK", sub { DeleteCallback($cw, @_); });
  $cw->RegisterCallback("ACTIVATE_CALLBACK",sub{ ActivateCallback($cw, @_); });
  $cw->RegisterCallback("CLEAR_CALLBACK", sub { ClearCallback($cw, @_); });
}

sub SelectionCallback {
  my ($self, $title, $sel, @args) = @_;
  my ($path, @rest) = $sel->get_selected_rows();
  if(defined $path){
	my $idx = $path->get_indices();
#	print "[$title] Path: $path idx: $idx\n";
	$self->SelectIndex( $idx, 'activate|callback');
  }else{
#	print "[$title] Path: undef\n"
  }
}

sub InsertCallback {
  my ($self, $data, @args) = @_;
  push @{$self->{data}}, [ $data->{LABEL} ];
}

sub ClearCallback {
  my ($self, @args) = @_;

#  print "Clearing list...\n";
  $self->set_data_array([]);
}

sub DeleteCallback {
  my ($self, $idx, @args) = @_;

#  print "Deleting index: $idx\n";
  splice(@{$self->{data}}, $idx, 1);
}

sub ActivateCallback {
  my ($self, $idx, @args) = @_;

  $self->select($idx);
}


sub UpdateCurrentSelection {
  my ($self, @args) = @_;

  my $e = $self->CurrentSelectionInfo();
  if(defined $e){
	my $idx = $self->CurrentSelection();
	$self->{data}->[$idx] = [ $e->{LABEL} ];
#	$self->SelectIndex( $idx, 'activate|callback'); 
  }
}



1;
