# Copyright (c) 1995-1999 Nick Ing-Simmons. All rights reserved.
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.

package Gtk2::Simple::ListBase;

our @ISA = qw/Gtk2::SimpleList/;

use Carp;
use Gtk2;
use Gtk2::SimpleList;

sub new {
	croak "Usage: $_[0]\->new (\\var, entry => value, ...)\n"
	    . " expecting a reference to the storage variable "
		. " and a list of entry and value pairs.\n"
	    . " can't create a SimpleGtk2:ListBase with no entries"
		unless @_ >= 3; # class, key1, val1

	return shift->new_from_widget (Gtk2::TreeView->new (), @_);
}

sub new_from_widget {
  my $class = shift;
  my $widg = shift;
  croak "TreeView is not a Gtk2::TreeView"
	  unless defined ($widg)
		  and UNIVERSAL::isa ($widg, 'Gtk2::TreeView');
  croak "Usage: $class\->new_from_widget (treelist, title, \\var, entry => val, ...)\n"
	  . " expecting a TreeView reference, list title, a reference to the storage"
	  . " for the selected index"
		  unless @_ >= 2; # ref, key1, val1
  my $title = shift;
  my $var = shift;
  my @entries = @_;

  my $result = Gtk2::SimpleList->new_from_treeview($widg, $title => 'text');
  bless $result, $class;

  $result->get_selection
	  ->signal_connect(changed => sub { $result->selection_changed_cb(@_); });

  foreach my $e (@entries){
	push @{$result->{data}}, [ $e ];
  }

  $result->update_selection_method(ref($var));

  return $result;
}

sub update_selection_method {
  my $self = shift;
  my $type = shift;
  # If boundVar is ref to array, we need to set multiple selection
  print "update_selection_method: [$type]\n";
  if( $type eq "ARRAY" ) {
	$self->get_selection->set_mode ('multiple');
  }else{
	$self->get_selection->set_mode ('single');
  }
}


sub selection_changed_cb {
  print "Derived classes need to redefine this.\n";
}



1;
