# Copyright (c) 1995-1999 Nick Ing-Simmons. All rights reserved.
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.

package Gtk2::Simple::Tie::List;

our @ISA = qw/Gtk2::Simple::ListBase/;

use Carp;
use Gtk2;
use Gtk2::Simple::ListBase;

sub new_from_widget {
  my ($class, $widg, $title, $var, @rest) = @_;

  my $result = Gtk2::Simple::ListBase->new_from_widget($widg, $title,
													 $var, @rest);
  bless $result, $class;

  $result->tie_var($var);
  return $result;
}

sub selection_changed_cb {}

sub tie_var {
  my $self = shift;
  my $var = shift;

  $self->untie_var();
  if(ref($var) eq 'ARRAY'){
	tie @$var, 'Gtk2::Simple::Tie::List::TiedList', $self;
  }else{
	tie $$var, 'Gtk2::Simple::Tie::List::TiedScalar', $self;
  }
  $self->update_selection_method(ref($var));
  $self->{boundVar} = $var;
}

sub untie_var {
  my $self = shift;

  if(defined $self->{boundVar}){
	my $ref = $self->{boundVar};
	my $val = $$ref;
	untie $$ref;
	$$ref = $val;
  }
}

sub DESTROY {
	my $self = shift;
	$self->untie_var();
	$self->SUPER::DESTROY();
}


########################
package Gtk2::Simple::Tie::List::TiedScalar;

use Gtk2;

sub TIESCALAR {
  my $pkg = shift;
  my $widg = shift;

  return bless { 'widg' => $widg }, $pkg;
}

sub FETCH {
  my $self = shift;

  my @idx = $self->{'widg'}->get_selected_indices();
  return $idx[0];
}

sub STORE {
  my $self = shift;
  my $value = shift;

  $self->{'widg'}->select($value);
}


###################################
package Gtk2::Simple::Tie::List::TiedList;

use strict;
use Gtk2;
use Carp;

=cut

TiedList is an array in which each element is the index of a multi-selection

=cut

sub TIEARRAY {
	my $pkg = shift;
	my $widg = shift;

	croak "usage tie (\@ary, 'class', widget)"
		unless $widg && UNIVERSAL::isa ($widg, 'Gtk2::Simple::Tie::List');

	my $ret = { 'widg' => $widg };
	return bless $ret, $pkg;
}

sub model {
  my $self = shift;
  return $self->{'widg'}->get_model();
}

sub selection {
  my $self = shift;
  return $self->{'widg'}->get_selection();
}

sub FETCH { # this, index
	my $iter = $_[0]->model()->iter_nth_child (undef, $_[1]);
	return undef unless defined $iter;
	return $_[0]->selection()->iter_is_selected($iter);
}

sub STORE { # this, index, value
  $_[0]->{'widg'}->select($_[1]);
  return 1;
}

sub FETCHSIZE { # this
  return $_[0]->model()->iter_n_children (undef);
}

sub PUSH { # this, list
  carp "PUSH: operation not supported"; 
}

sub POP { # this
  carp "POP: operation not supported"; 
}

sub SHIFT { # this
  carp "SHIFT: operation not supported"; 
}

sub UNSHIFT { # this, list
  carp "UNSHIFT: operation not supported"; 
}

# note: really, arrays aren't supposed to support the delete operator this
#       way, but we don't want to break existing code.
sub DELETE { # this, key
  carp "DELETE: operation not supported"; 
}

sub CLEAR { # this
  $_[0]->selection()->unselect_all();
}

# note: arrays aren't supposed to support exists, either.
sub EXISTS { # this, key
  return( $_[1] < $_[0]->model()->iter_n_children );
}

# we can't really, reasonably, extend the tree store in one go, it will be 
# extend as items are added
sub EXTEND {  
  carp "EXTEND: operation not supported"; 
}

sub STORESIZE { 
carp "STORESIZE: operation not supported"; 
}

sub SPLICE { # this, offset, length, list
  carp "SPLICE: operation not supported"; 
}


1;
